import { get } from 'https';
import { createHash } from 'crypto';

export interface GetUsernameUuidOptions {
	username: string;
	onlineMode?: boolean;
}

export interface GetUsernameUuid {
	(options: GetUsernameUuidOptions): Promise<string>;
}

export class InvalidArgumentTypeError extends TypeError {
	constructor(message: string) {
		super(message);
		this.name = 'InvalidArgumentTypeError';
	}
}

const formatUuidBytes = (bytes: Buffer) => {
	return [
		bytes.slice(0, 4),
		bytes.slice(4, 6),
		bytes.slice(6, 8),
		bytes.slice(8, 10),
		bytes.slice(10)
	]
		.map((b) => b.toString('hex'))
		.join('-');
};

export const getUsernameUuid: GetUsernameUuid = async (options: GetUsernameUuidOptions): Promise<string> => {
	const { username, onlineMode = false } = options as unknown as Record<string, unknown>;

	if (typeof username !== 'string') {
		throw new InvalidArgumentTypeError('Username must be a string');
	}

	if (!onlineMode) {
		const md5 = createHash('md5');
		md5.update(Buffer.from(`OfflinePlayer:${username}`, 'utf-8'));
		const bytes = md5.digest();

		bytes[6] &= 0x0f;
		bytes[6] |= 0x30;

		bytes[8] &= 0x3f;
		bytes[8] |= 0x80;

		return formatUuidBytes(bytes);
	}

	return new Promise((resolve, reject) => {
		get(`https://api.mojang.com/users/profiles/minecraft/${encodeURIComponent(username)}`, res => {
			let uuid = '';

			res.on('data', (c) => {
				const json = c.toString('utf-8');
				const data = JSON.parse(json);
				const bytes = Buffer.from(data.id, 'hex');

				uuid = formatUuidBytes(bytes);
			});

			res.on('end', () => {
				resolve(uuid);
			});
		})
			.on('error', reject);
	});
};
